// main.c, created by James Rishe and Andrew Melkonian
//  using main.c from Nortsam as a basis

#include "xparameters.h"
#include "xbasic_types.h"
#include "xio.h"
#include "xintc_l.h"
#include "xuartlite_l.h"
#include "fft_settings.h"

#define FONT_OFFSET 0xA00

#define RAM_H 80
#define RAM_V 60

#define VGA_START 0x00800000
#define AUDIO_START 0xFEFE0000
#define LAST_LINE 59

#define _A 1
#define _B 2
#define _C 3
#define _D 4
#define _E 5
#define _F 6
#define _G 7
#define _0 8
#define _1 9
#define _2 10
#define _3 11
#define _4 12
#define _5 13
#define _6 14
#define _7 15
#define _8 16
#define _9 17
#define _m 18
#define _M 19
#define _P 20
#define _s 21
#define _fl 22
#define XPAR_INTC_AK4565_INTERRUPT_INTR 0


// defined in isr.c
extern void audio_intr_handler(void *callback);
extern volatile Xuint16 bufInUse[];
extern volatile Xuint16 bufReady[];

Xint32 audioBuffer[ FFT_SIZE << 1 ];
Xint16 audioBufferLEFT[ FFT_SIZE << 1 ];

Xint32 *astart;
Xint16 *startLPtr, *startRPtr;


/*
 * setup_interrupts: Initialize the interrupt sources and handlers
 */
void setup_interrupts()
{
  /*
   * Reset the interrupt controller peripheral
   */

  /* Disable the interrupt signal */
  XIntc_mMasterDisable(XPAR_INTC_SINGLE_BASEADDR);
  /* Disable all interrupt sources */
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR,0);
  /* Acknowledge all possible interrupt sources
     to make sure none are pending */
  XIntc_mAckIntr(XPAR_INTC_SINGLE_BASEADDR, 0xffffffff);

  /*
   * Install the codec interrupt handler
   */
  XIntc_InterruptVectorTable[XPAR_INTC_AK4565_INTERRUPT_INTR].Handler =
    audio_intr_handler;

  // enable interrupt sources
  /* Enable CPU interrupts */
  microblaze_enable_interrupts();
  /* Enable interrupts from the interrupt controller */
  XIntc_mMasterEnable(XPAR_INTC_SINGLE_BASEADDR);
  /* Tell the interrupt controller to accept interrupts from the codec */
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, XPAR_AK4565_INTERRUPT_MASK);
}



/* Address of a particular character on the screen (rows are 80) */
#define CHAR(r,c) \
   (((unsigned char *)(XPAR_XSB300_BASEADDR))[(c) + ((r) << 6) + ((r) << 4)])

/* Start of font memory */
#define FONT ((unsigned char *)XPAR_XSB300_BASEADDR + FONT_OFFSET)

/* Cursor position on the screen */
int x = 0;
int y = 0;

/* Place a single character ch on the screen at position (column, row) */
void put_char(char ch, int row, int column)
{
  CHAR(row, column) = ch;
}


int main()
{
  int i;
  int j,k,l;
  Xint32 maxmag, maxind;
  volatile unsigned char *p;
  Xuint32 data;
  // Enable the instruction cache: makes the code run 6 times faster
  microblaze_enable_icache();

  setup_interrupts();
  l=0;

  // Clear  the  relevant //
  // positions of the RAM //
  // i.e., the first 4800 //
  for(x = 0 ; x < RAM_H*RAM_V; x++)
    XIo_Out8(VGA_START+x, 0);

  // put static words on the screen
  setup_graphics();

  // this never changes, define outside loop.  
  startLPtr = (Xint16*)(audioBufferLEFT)-1;


  i = 0;
  for ( ;; ) {
    // *******************************
    // double buffer logistics

    while (bufReady[i] == 0) 
      print(""); // need this delay, a busy-wait loop prevents isr context switch?!
    bufInUse[(i+1) % 2] = 0;
    bufInUse[i] = 1;

    // fudge starting pointers to account for FFT starting
    //  with index 1.
    astart = (Xint32*)(&(audioBuffer[ (i<<BUFFER_SHIFT_OFFSET) ])) - 1;


    separate(astart, startLPtr);
    //    putnum(startLPtr[5]);



    bitReverse( startLPtr );
    //    putnum(startLPtr[5]);
    // after this point:
    //   audioBuffer[] holds bit-rearranged LEFT samples, MSB=real/LSB=imag


    doFFT( startLPtr );
    /*    maxind = 0; 
    maxmag = 0;
    for(i = 0; i < FFT_SIZE; i++){
      if(startLPtr[i] > maxmag){
	maxind = i;
	maxmag = startLPtr[i];
      }
    }
    print("\r\nMax value: ");
    putnum(maxmag);
    print("  Max index:");
    putnum(maxind);*/
    //    if(l == 15){
    visualize(startLPtr);
    //      l = 0;
    //    }
    l++;
    i = (i+1) & 1;
  }
  
  print("Goodbye\r\n");
  return 0;
}
